@@ -1,9 +1,10 @@
 import json
+import time
 import unittest
 
 from flask import Flask, session, request, json as flask_json
 from flask_socketio import SocketIO, send, emit, join_room, leave_room, \
-    Namespace, disconnect
+    Namespace, disconnect, ConnectionRefusedError
 
 app = Flask(__name__)
 app.config['SECRET_KEY'] = 'secret'
@@ -16,7 +17,7 @@ def on_connect(auth):
     if auth != {'foo': 'bar'}:  # pragma: no cover
         return False
     if request.args.get('fail'):
-        return False
+        raise ConnectionRefusedError('failed!')
     send('connected')
     send(json.dumps(request.args.to_dict(flat=False)))
     send(json.dumps({h: request.headers[h] for h in request.headers.keys()
@@ -25,7 +26,7 @@ def on_connect(auth):
 
 
 @socketio.on('disconnect')
-def on_disconnect():
+def on_disconnect(reason):
     global disconnected
     disconnected = '/'
 
@@ -44,6 +45,14 @@ def on_disconnect_test():
     disconnected = '/test'
 
 
+@socketio.on('connect', namespace='/bgtest')
+def on_bgtest_connect():
+    def background_task():
+        socketio.emit('bgtest', namespace='/bgtest')
+
+    socketio.start_background_task(background_task)
+
+
 @socketio.event
 def message(message):
     send(message)
@@ -99,6 +108,11 @@ def get_request_event2(data):
 socketio.on_event('yet another custom event', get_request_event2)
 
 
+@socketio.on('*')
+def catch_all(event, data):
+    emit('my custom response', (event, data))
+
+
 @socketio.on('my custom namespace event', namespace='/test')
 def on_custom_event_test(data):
     emit('my custom namespace response', data, namespace='/test')
@@ -173,6 +187,11 @@ def on_changing_response():
     return data
 
 
+@socketio.on('wildcard', namespace='*')
+def wildcard(data):
+    emit('my custom response', data)
+
+
 @socketio.on_error()
 def error_handler(value):
     if isinstance(value, AssertionError):
@@ -420,13 +439,34 @@ class TestSocketIO(unittest.TestCase):
     def test_emit_binary(self):
         client = socketio.test_client(app, auth={'foo': 'bar'})
         client.get_received()
-        client.emit('my custom event', {u'a': b'\x01\x02\x03'})
+        client.emit('my custom event', {'a': b'\x01\x02\x03'})
         received = client.get_received()
         self.assertEqual(len(received), 1)
         self.assertEqual(len(received[0]['args']), 1)
         self.assertEqual(received[0]['name'], 'my custom response')
         self.assertEqual(received[0]['args'][0]['a'], b'\x01\x02\x03')
 
+    def test_emit_catch_all_event(self):
+        client = socketio.test_client(app, auth={'foo': 'bar'})
+        client.get_received()
+        client.emit('random event', {'foo': 'bar'})
+        received = client.get_received()
+        self.assertEqual(len(received), 1)
+        self.assertEqual(len(received[0]['args']), 2)
+        self.assertEqual(received[0]['name'], 'my custom response')
+        self.assertEqual(received[0]['args'][0], 'random event')
+        self.assertEqual(received[0]['args'][1], {'foo': 'bar'})
+
+    def test_send_catch_all_namespace(self):
+        client = socketio.test_client(app, namespace='/test')
+        client.get_received('/test')
+        client.emit('wildcard', {'a': 'b'}, namespace='/test')
+        received = client.get_received('/test')
+        self.assertEqual(len(received), 1)
+        self.assertEqual(len(received[0]['args']), 1)
+        self.assertEqual(received[0]['name'], 'my custom response')
+        self.assertEqual(received[0]['args'][0]['a'], 'b')
+
     def test_request_event_data(self):
         client = socketio.test_client(app, auth={'foo': 'bar'})
         client.get_received()
@@ -762,6 +802,14 @@ class TestSocketIO(unittest.TestCase):
         self.assertEqual(received[0]['args'][0], {'foo': 'bar'})
         self.assertEqual(ack, {'foo': 'baz'})
 
+    def test_background_task(self):
+        client = socketio.test_client(app, namespace='/bgtest')
+        self.assertTrue(client.is_connected(namespace='/bgtest'))
+        time.sleep(0.1)
+        received = client.get_received('/bgtest')
+        self.assertEqual(len(received), 1)
+        self.assertEqual(received[0]['name'], 'bgtest')
+
 
 if __name__ == '__main__':
     unittest.main()
